﻿  using System;

using PT.Scheduler;
using PT.Scheduler.Simulation;
using PT.Scheduler.Simulation.Customizations;

namespace PlanetTogether.Sample.LearningCurve
{
    public class LearningCurve : PT.Scheduler.Simulation.Customizations.RequiredCapacityCustomization
    {
        public override string Name => "Learning Curve";
        public override string Description => "Product learning curve. Products are produced faster over time based on Item UDfs";

        /// <summary>
        /// This class stores and tracks all item learning curve data as well as the current scheduled counts
        /// </summary>
        private LearningCurveData m_data;

        /// <summary>
        /// Method that overrides setup
        /// </summary>
        protected override void AfterRequiredCapacityCalculation(BaseActivity a_activity, BaseResource a_resource, RequiredCapacityChangeHelper a_rc, long a_startTicks)
        {
            if (a_activity is InternalActivity act)  
            {
                if (act.Operation.Products.Count > 0)
                {
                    decimal modifier = m_data.GetRateModifier(act.Operation.Products[0].Item.Id);
                    if (modifier != 0)
                    {
                        //There is a modifier tracked. Updated the processing time
                        //This sample multiplies the standard rate by the multiplier.

                        //TODO: For your customization, decide whether the standard run rate is the current slow rate, or the goal speed.
                        //If using multiple people/resources, the goal speed is probably easier.
                        //This sample uses the standard as the goal, so the modifier will return 1 as the minimum value

                        TimeSpan modifiedRate = new TimeSpan((long)Math.Ceiling(a_rc.RequiredCapacityChanger.ProcessingSpan.TimeSpanTicks * modifier));
                        a_rc.RequiredCapacityChanger.ProcessingSpan = new RequiredSpan(modifiedRate.Ticks, false);
                    }
                }
            }
        }

        /// <summary>
        /// Synchronize the data from another customization point
        /// </summary>
        /// <param name="a_data"></param>
        internal void SetLearningCurveData(LearningCurveData a_data)
        {
            m_data = a_data;
        }
    }
}
