﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using PT.Scheduler;

namespace PlanetTogether.Sample.LearningCurve
{
    internal class LearningCurveData
    {
        readonly Dictionary<BaseId, ProductLearningCurve> m_productLearningCurves = new Dictionary<BaseId, ProductLearningCurve>();
        private const string c_udfFieldName = "LearningCurveRate";

        internal LearningCurveData(ScenarioDetail a_sd)
        {
            foreach (Item item in a_sd.ItemManager)
            {
                if (item.UserFields?.Find(c_udfFieldName)?.DataValue is decimal learningCurveRate)
                {
                    m_productLearningCurves.Add(item.Id, new ProductLearningCurve(item));
                }
            }
        }

        /// <summary>
        /// Track that an operation scheduled so the efficiency modifier can be updated for the next operation
        /// </summary>
        internal void OperationScheduled(InternalOperation a_op)
        {
            //TODO: Resource can be looked up from a_op scheduled resources
            if (a_op.Products.Count > 0)
            {
                if (m_productLearningCurves.TryGetValue(a_op.Products[0].Item.Id, out ProductLearningCurve data))
                {
                    data.OperationScheduled(a_op);
                }
            }
        }

        public decimal GetRateModifier(BaseId a_itemId)
        {
            if (m_productLearningCurves.TryGetValue(a_itemId, out ProductLearningCurve data))
            {
                return data.GetRateModifier();
            }

            //Return default invalid value
            return 0m;
        }
    }

    internal class ProductLearningCurve
    {
        private int m_scheduledCount;
        private decimal m_rateChange;
        private decimal m_startingEfficiency;
        private const string c_udfFieldName = "LearningCurveRate";

        internal ProductLearningCurve(Item a_item)
        {
            if (a_item.UserFields.Find(c_udfFieldName)?.DataValue is decimal learningCurveRate)
            {
                m_rateChange = learningCurveRate;
            }

            //TODO: Read other properties from additional UDFs
            //This sample starts at 50% slower than standard
            m_startingEfficiency = 1.5m;
        }

        internal void OperationScheduled(InternalOperation a_op)
        {
            m_scheduledCount++;
        }

        public decimal GetRateModifier()
        {
            //TODO: Based on parameters, return the appropriate rate modifier for this operation
            //This sample returns the starting efficiency + the rate change per scheduled operation and limits the rate to 100% of the standard
            decimal rate = m_startingEfficiency - (m_scheduledCount * m_rateChange);
            return Math.Max(1, rate); //Limit to 100% of the standard
        }
    }
}
