﻿using System;

using PT.Scheduler;
using PT.Scheduler.Simulation.Customizations;

namespace PlanetTogether.Sample.LearningCurve
{
    internal class OperationScheduled : PT.Scheduler.Simulation.Customizations.OperationScheduled, ICustomizationRequiresInitialization
    {
        public override string Name => "Learning Curve 2";
        public override string Description => "Part of the learning curve add-in";

        /// <summary>
        /// This class stores and tracks all item learning curve data as well as the current scheduled counts
        /// This customization point keeps track of creating and sharing the reference
        /// </summary>
        private LearningCurveData m_data;

        //The required capacity customization point in this project.
        //We can't simply create one here, because it needs to be shared across the customization points. 
        //The RestoreReferences function below will 
        private LearningCurve m_learningCurveCustomization;

        protected override void SimulationInitialization(ScenarioDetail a_sd)
        {
            //Initialize the learning curve data from the current model
            m_data = new LearningCurveData(a_sd);
            //Send to the other customization point that doesn't have a SimulationInitialization override
            m_learningCurveCustomization?.SetLearningCurveData(m_data);
        }

        protected override void Scheduled(InternalOperation a_io, ref ChangeableOperationValues r_changableOperationValues, ScenarioDetail a_sd, long a_simClock)
        {
            m_data.OperationScheduled(a_io);
        }

        public void InitializeAfterCreation(ScenarioDetail a_sd)
        {
            //Nothing to do here
        }

        /// <summary>
        /// After all customizations are loaded, restore the reference to the required capacity customization point
        /// </summary>
        /// <param name="a_sd"></param>
        public void RestoreReferences(ScenarioDetail a_sd)
        {
            if (a_sd.CustomizationInstances?.SchedulerCustomization is LearningCurve learningCurveCustomization)
            {
                m_learningCurveCustomization = learningCurveCustomization;
            }
        }
    }
}
